# -----------------------------------------------------------
# Copyright (c) 2023 Lauris BH
# SPDX-License-Identifier: MIT
# -----------------------------------------------------------

import click
import dataclasses
import json
import logging

from karcher.exception import KarcherHomeException
from karcher.karcher import KarcherHome
from karcher.consts import Region

try:
    from rich import print as echo
except ImportError:
    echo = click.echo


class EnhancedJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if dataclasses.is_dataclass(o):
            return dataclasses.asdict(o)
        return super().default(o)


class GlobalContextObject:
    def __init__(
            self,
            debug: int = 0,
            output: str = 'json',
            region: Region = Region.EU):
        self.debug = debug
        self.output = output
        self.region = region

    def print(self, result):
        data_variable = getattr(result, 'data', None)
        if data_variable is not None:
            result = data_variable
        if self.output == 'json_pretty':
            echo(json.dumps(result, cls=EnhancedJSONEncoder, indent=4))
        else:
            echo(json.dumps(result, cls=EnhancedJSONEncoder))


@click.group()
@click.option('-d', '--debug', is_flag=True, help='Enable debug mode.')
@click.option(
    '-o',
    '--output',
    type=click.Choice(['json', 'json_pretty']),
    default='json',
    help='Output format. Default: "json"')
@click.option(
    '-r',
    '--region',
    type=click.Choice([Region.EU, Region.US, Region.CN]),
    default=Region.EU,
    help='Region of the server to query. Default: "eu"')
@click.pass_context
def cli(ctx: click.Context, debug: int, output: str, region: Region):
    """Tool for connectiong and getting information from Kärcher Home Robots."""
    level = logging.INFO
    if debug > 0:
        level = logging.DEBUG

    logging.basicConfig(level=level)

    ctx.obj = GlobalContextObject(debug=debug, output=output, region=region)


def safe_cli():
    try:
        cli()
    except KarcherHomeException as ex:
        echo(json.dumps({
            'code': ex.code,
            'message': ex.message
        }))
        return


@cli.command()
@click.pass_context
def urls(ctx: click.Context):
    """Get region information."""

    kh = KarcherHome(region=ctx.obj.region)
    d = kh.get_urls()

    ctx.obj.print(d)


@cli.command()
@click.option('--username', '-u', help='Username to login with.')
@click.option('--password', '-p', help='Password to login with.')
@click.pass_context
def login(ctx: click.Context, username: str, password: str):
    """Get user session tokens."""

    kh = KarcherHome(region=ctx.obj.region)
    ctx.obj.print(kh.login(username, password))


@cli.command()
@click.option('--username', '-u', default=None, help='Username to login with.')
@click.option('--password', '-p', default=None, help='Password to login with.')
@click.option('--auth-token', '-t', default=None, help='Authorization token.')
@click.pass_context
def devices(ctx: click.Context, username: str, password: str, auth_token: str):
    """List all devices."""

    kh = KarcherHome(region=ctx.obj.region)
    if auth_token is not None:
        kh.login_token(auth_token, '')
    elif username is not None and password is not None:
        kh.login(username, password)
    else:
        raise click.BadParameter(
            'Must provide either token or username and password.')

    devices = kh.get_devices()

    # Logout if we used a username and password
    if auth_token is None:
        kh.logout()

    ctx.obj.print(devices)


@cli.command()
@click.option('--username', '-u', default=None, help='Username to login with.')
@click.option('--password', '-p', default=None, help='Password to login with.')
@click.option('--auth-token', '-t', default=None, help='Authorization token.')
@click.option('--mqtt-token', '-m', default=None, help='MQTT authorization token.')
@click.option('--device-id', '-d', required=True, help='Device ID.')
@click.pass_context
def device_properties(
        ctx: click.Context,
        username: str,
        password: str,
        auth_token: str,
        mqtt_token: str,
        device_id: str):
    """Get device properties."""

    kh = KarcherHome(region=ctx.obj.region)
    if auth_token is not None:
        kh.login_token(auth_token, mqtt_token)
    elif username is not None and password is not None:
        kh.login(username, password)
    else:
        raise click.BadParameter(
            'Must provide either token or username and password.')

    dev = None
    for device in kh.get_devices():
        if device.device_id == device_id:
            dev = device
            break

    if dev is None:
        raise click.BadParameter('Device ID not found.')

    props = kh.get_device_properties(dev)

    # Logout if we used a username and password
    if auth_token is None:
        kh.logout()

    ctx.obj.print(props)



# -----------------------------------------------------------
# Copyright (c) 2023 Lauris BH
# SPDX-License-Identifier: MIT
# -----------------------------------------------------------

from google.protobuf.json_format import MessageToDict

from . import mapdata_pb2
from .utils import snake_case, snake_case_fields


class Map:
    @staticmethod
    def parse(data: bytes):
        m = Map()
        rm = mapdata_pb2.RobotMap()
        rm.ParseFromString(data)
        d = {}
        for k, v in MessageToDict(rm).items():
            if k == 'mapData':
                v = v['mapData']
            v = snake_case_fields(v)
            d[snake_case(k)] = v
        setattr(m, 'data', d)
        return m



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mapdata.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rmapdata.proto\x12\x07karcher\"[\n\nMapExtInfo\x12\x15\n\rtaskBeginDate\x18\x01 \x01(\x05\x12\x15\n\rmapUploadDate\x18\x02 \x01(\x05\x12\x10\n\x08mapValid\x18\x03 \x01(\x05\x12\r\n\x05\x61ngle\x18\x04 \x01(\x02\"\x8a\x01\n\x0bMapHeadInfo\x12\x11\n\tmapHeadId\x18\x01 \x01(\x05\x12\r\n\x05sizeX\x18\x02 \x01(\x05\x12\r\n\x05sizeY\x18\x03 \x01(\x05\x12\x0c\n\x04minX\x18\x04 \x01(\x02\x12\x0c\n\x04minY\x18\x05 \x01(\x02\x12\x0c\n\x04maxX\x18\x06 \x01(\x02\x12\x0c\n\x04maxY\x18\x07 \x01(\x02\x12\x12\n\nresolution\x18\x08 \x01(\x02\"\x1e\n\x0bMapDataInfo\x12\x0f\n\x07mapData\x18\x01 \x01(\x0c\"0\n\nAllMapInfo\x12\x11\n\tmapHeadId\x18\x01 \x01(\x05\x12\x0f\n\x07mapName\x18\x02 \x01(\t\"@\n\x18\x44\x65viceCoverPointDataInfo\x12\x0e\n\x06update\x18\x01 \x01(\x05\x12\t\n\x01x\x18\x02 \x01(\x02\x12\t\n\x01y\x18\x03 \x01(\x02\"Z\n\x15\x44\x65viceHistoryPoseInfo\x12\x0e\n\x06poseId\x18\x01 \x01(\x05\x12\x31\n\x06points\x18\x02 \x03(\x0b\x32!.karcher.DeviceCoverPointDataInfo\"7\n\x12\x44\x65vicePoseDataInfo\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\x0b\n\x03phi\x18\x03 \x01(\x02\"Z\n\x15\x44\x65viceCurrentPoseInfo\x12\x0e\n\x06poseId\x18\x01 \x01(\x05\x12\x0e\n\x06update\x18\x02 \x01(\x05\x12\t\n\x01x\x18\x03 \x01(\x02\x12\t\n\x01y\x18\x04 \x01(\x02\x12\x0b\n\x03phi\x18\x05 \x01(\x02\"\'\n\x0f\x44\x65vicePointInfo\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\"o\n\x12\x44\x65viceAreaDataInfo\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0c\n\x04type\x18\x02 \x01(\x05\x12\x11\n\tareaIndex\x18\x03 \x01(\x05\x12(\n\x06points\x18\x04 \x03(\x0b\x32\x18.karcher.DevicePointInfo\"v\n\x1d\x44\x65viceNavigationPointDataInfo\x12\x0f\n\x07pointId\x18\x01 \x01(\x05\x12\x0e\n\x06status\x18\x02 \x01(\x05\x12\x11\n\tpointType\x18\x03 \x01(\x05\x12\t\n\x01x\x18\x04 \x01(\x02\x12\t\n\x01y\x18\x05 \x01(\x02\x12\x0b\n\x03phi\x18\x06 \x01(\x02\"g\n\x17\x43leanPerferenceDataInfo\x12\x11\n\tcleanMode\x18\x01 \x01(\x05\x12\x12\n\nwaterLevel\x18\x02 \x01(\x05\x12\x11\n\twindPower\x18\x03 \x01(\x05\x12\x12\n\ntwiceClean\x18\x04 \x01(\x05\"\x8f\x02\n\x0cRoomDataInfo\x12\x0e\n\x06roomId\x18\x01 \x01(\x05\x12\x10\n\x08roomName\x18\x02 \x01(\t\x12\x12\n\nroomTypeId\x18\x03 \x01(\x05\x12\x12\n\nmeterialId\x18\x04 \x01(\x05\x12\x12\n\ncleanState\x18\x05 \x01(\x05\x12\x11\n\troomClean\x18\x06 \x01(\x05\x12\x16\n\x0eroomCleanIndex\x18\x07 \x01(\x05\x12.\n\x0croomNamePost\x18\x08 \x01(\x0b\x32\x18.karcher.DevicePointInfo\x12\x35\n\x0b\x63leanPerfer\x18\t \x01(\x0b\x32 .karcher.CleanPerferenceDataInfo\x12\x0f\n\x07\x63olorId\x18\n \x01(\x05\"\"\n\x10\x44\x65viceRoomMatrix\x12\x0e\n\x06matrix\x18\x01 \x01(\x0c\"?\n\x18\x44\x65viceChainPointDataInfo\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x12\r\n\x05value\x18\x03 \x01(\x05\"\\\n\x17\x44\x65viceRoomChainDataInfo\x12\x0e\n\x06roomId\x18\x01 \x01(\x05\x12\x31\n\x06points\x18\x02 \x03(\x0b\x32!.karcher.DeviceChainPointDataInfo\"\x80\x01\n\x0eObjectDataInfo\x12\x10\n\x08objectId\x18\x01 \x01(\x05\x12\x14\n\x0cobjectTypeId\x18\x02 \x01(\x05\x12\x12\n\nobjectName\x18\x03 \x01(\t\x12\x0f\n\x07\x63onfirm\x18\x04 \x01(\x05\x12\t\n\x01x\x18\x05 \x01(\x02\x12\t\n\x01y\x18\x06 \x01(\x02\x12\x0b\n\x03url\x18\x07 \x01(\t\"f\n\x11\x46urnitureDataInfo\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0e\n\x06typeId\x18\x02 \x01(\x05\x12(\n\x06points\x18\x03 \x03(\x0b\x32\x18.karcher.DevicePointInfo\x12\x0b\n\x03url\x18\x04 \x01(\t\"q\n\tHouseInfo\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x63urMapCount\x18\x03 \x01(\x05\x12\x12\n\nmaxMapSize\x18\x04 \x01(\x05\x12!\n\x04maps\x18\x05 \x03(\x0b\x32\x13.karcher.AllMapInfo\"\x91\x06\n\x08RobotMap\x12\x0f\n\x07mapType\x18\x01 \x01(\x05\x12\'\n\nmapExtInfo\x18\x02 \x01(\x0b\x32\x13.karcher.MapExtInfo\x12%\n\x07mapHead\x18\x03 \x01(\x0b\x32\x14.karcher.MapHeadInfo\x12%\n\x07mapData\x18\x04 \x01(\x0b\x32\x14.karcher.MapDataInfo\x12$\n\x07mapInfo\x18\x05 \x03(\x0b\x32\x13.karcher.AllMapInfo\x12\x33\n\x0bhistoryPose\x18\x06 \x01(\x0b\x32\x1e.karcher.DeviceHistoryPoseInfo\x12\x32\n\rchargeStation\x18\x07 \x01(\x0b\x32\x1b.karcher.DevicePoseDataInfo\x12\x33\n\x0b\x63urrentPose\x18\x08 \x01(\x0b\x32\x1e.karcher.DeviceCurrentPoseInfo\x12\x31\n\x0cvirtualWalls\x18\t \x03(\x0b\x32\x1b.karcher.DeviceAreaDataInfo\x12.\n\tareasInfo\x18\n \x03(\x0b\x32\x1b.karcher.DeviceAreaDataInfo\x12@\n\x10navigationPoints\x18\x0b \x03(\x0b\x32&.karcher.DeviceNavigationPointDataInfo\x12+\n\x0croomDataInfo\x18\x0c \x03(\x0b\x32\x15.karcher.RoomDataInfo\x12-\n\nroomMatrix\x18\r \x01(\x0b\x32\x19.karcher.DeviceRoomMatrix\x12\x33\n\troomChain\x18\x0e \x03(\x0b\x32 .karcher.DeviceRoomChainDataInfo\x12(\n\x07objects\x18\x0f \x03(\x0b\x32\x17.karcher.ObjectDataInfo\x12\x31\n\rfurnitureInfo\x18\x10 \x03(\x0b\x32\x1a.karcher.FurnitureDataInfo\x12&\n\nhouseInfos\x18\x11 \x03(\x0b\x32\x12.karcher.HouseInfob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mapdata_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_MAPEXTINFO']._serialized_start=26
  _globals['_MAPEXTINFO']._serialized_end=117
  _globals['_MAPHEADINFO']._serialized_start=120
  _globals['_MAPHEADINFO']._serialized_end=258
  _globals['_MAPDATAINFO']._serialized_start=260
  _globals['_MAPDATAINFO']._serialized_end=290
  _globals['_ALLMAPINFO']._serialized_start=292
  _globals['_ALLMAPINFO']._serialized_end=340
  _globals['_DEVICECOVERPOINTDATAINFO']._serialized_start=342
  _globals['_DEVICECOVERPOINTDATAINFO']._serialized_end=406
  _globals['_DEVICEHISTORYPOSEINFO']._serialized_start=408
  _globals['_DEVICEHISTORYPOSEINFO']._serialized_end=498
  _globals['_DEVICEPOSEDATAINFO']._serialized_start=500
  _globals['_DEVICEPOSEDATAINFO']._serialized_end=555
  _globals['_DEVICECURRENTPOSEINFO']._serialized_start=557
  _globals['_DEVICECURRENTPOSEINFO']._serialized_end=647
  _globals['_DEVICEPOINTINFO']._serialized_start=649
  _globals['_DEVICEPOINTINFO']._serialized_end=688
  _globals['_DEVICEAREADATAINFO']._serialized_start=690
  _globals['_DEVICEAREADATAINFO']._serialized_end=801
  _globals['_DEVICENAVIGATIONPOINTDATAINFO']._serialized_start=803
  _globals['_DEVICENAVIGATIONPOINTDATAINFO']._serialized_end=921
  _globals['_CLEANPERFERENCEDATAINFO']._serialized_start=923
  _globals['_CLEANPERFERENCEDATAINFO']._serialized_end=1026
  _globals['_ROOMDATAINFO']._serialized_start=1029
  _globals['_ROOMDATAINFO']._serialized_end=1300
  _globals['_DEVICEROOMMATRIX']._serialized_start=1302
  _globals['_DEVICEROOMMATRIX']._serialized_end=1336
  _globals['_DEVICECHAINPOINTDATAINFO']._serialized_start=1338
  _globals['_DEVICECHAINPOINTDATAINFO']._serialized_end=1401
  _globals['_DEVICEROOMCHAINDATAINFO']._serialized_start=1403
  _globals['_DEVICEROOMCHAINDATAINFO']._serialized_end=1495
  _globals['_OBJECTDATAINFO']._serialized_start=1498
  _globals['_OBJECTDATAINFO']._serialized_end=1626
  _globals['_FURNITUREDATAINFO']._serialized_start=1628
  _globals['_FURNITUREDATAINFO']._serialized_end=1730
  _globals['_HOUSEINFO']._serialized_start=1732
  _globals['_HOUSEINFO']._serialized_end=1845
  _globals['_ROBOTMAP']._serialized_start=1848
  _globals['_ROBOTMAP']._serialized_end=2633
# @@protoc_insertion_point(module_scope)



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='karcher-home',
    packages=['karcher'],
    include_package_data=True,
    version='0.3',
    license='MIT',
    description='Kärcher Home Robots client',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Lauris BH',
    author_email='lauris@nix.lv',
    url='https://github.com/lafriks/python-karcher',
    download_url='https://github.com/lafriks/python-karcher/releases/download/v0.3/karcher-home-0.3.tar.gz',
    platforms='any',
    install_requires=[
        'requests',
        'tzlocal',
        'click',
        'pycryptodome',
        'protobuf'
    ],
    entry_points='''
        [console_scripts]
        karcher-home=karcher.cli:safe_cli
    ''',
)
